<?php

namespace Drupal\email_message\Plugin\Field\FieldType;

use Drupal\Component\Utility\Random;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\text\Plugin\Field\FieldType\TextLongItem;

/**
 * Defines the 'email_message' field type.
 *
 * @FieldType(
 *   id = "email_message",
 *   label = @Translation("Email message"),
 *   description = @Translation("Field to store an email message (email subject & body."),
 *   default_widget = "email_message_default",
 *   default_formatter = "email_message_default_formatter"
 * )
 */
class EmailMessageItem extends TextLongItem {

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition): array {
    // Use the 'text_long' field schema for the email body properties.
    $schema = parent::schema($field_definition);
    $schema['columns']['subject'] = [
      'type' => 'text',
      'size' => 'normal',
    ];
    return $schema;
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition): array {
    // Use the 'text_long' field property definition for the email body
    // properties.
    $properties = parent::propertyDefinitions($field_definition);
    $properties['subject'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Email subject'))
      ->setRequired(TRUE);
    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition): array {
    $values = parent::generateSampleValue($field_definition);
    $random = new Random();
    $values['subject'] = $random->string(100);
    return $values;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty(): bool {
    $body_empty = parent::isEmpty();
    return $body_empty || ($this->subject === NULL || $this->subject === '');
  }

  /**
   * Get the email message subject.
   *
   * @return string
   *   The email message subject.
   */
  public function getEmailSubject(): string {
    return $this->get('subject')->getString();
  }

  /**
   * Get the email message body.
   *
   * @return string
   *   Email message body.
   */
  public function getEmailBody(): string {
    return $this->get('processed')->getString();
  }

}
